# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/nc-to-thredds/source/ProcessData.py $
# Last committed:     $Revision: 411652 $; $Author: vvuik $; $Date: 2019-11-08 11:15:26 +0100 (Fri, 08 Nov 2019) $
# -*- coding: utf-8 -*-
"""
Created on Mon Sep  9 15:48:42 2019

@author: vuik
"""

from datetime import datetime, timedelta
import pandas as pd
import numpy as np

from importChecks import import_checks
from getData import get_parameters_checks, get_location_list
    
#%%
    
def fews_params(ncpar):
    
    return ncpar.replace('_', '.')

#%%

def doChecks(instrument, settings, ndays, specificlocs=None):
    
    startTime = datetime.now().replace(microsecond=0,second=0,minute=0,hour=0) - timedelta(days=ndays)
    endTime = startTime + timedelta(days=1)
    
    print("Times {}".format(startTime))
    print("Times {}".format(endTime))
    
    daystr = str(datetime.date(startTime))
    daystr = daystr.replace('-','')
    
    # get parameters checks
    parametersC = get_parameters_checks(instrument)
    
    # get expected numbers from file
    expectedN = pd.read_csv(settings['nrfile'],sep=';',names=['instrument','N'])
    N = float(expectedN.loc[expectedN['instrument']==instrument,'N'])
    
    if specificlocs is None:
        location_list = get_location_list(instrument, startTime)
    else:
        location_list = specificlocs
    
    li = []
    
    for location_id in location_list:
        
        print("Processing {}".format(location_id) + ', ' + instrument)
        
        # Import data
        ts, location_name = import_checks(instrument, location_id, parametersC, startTime, endTime, settings)
        
        # No timeseries, then continue with next location
        if ts is None:
            continue
        
        # Delete endTime
        try:
            ts.drop(pd.Timestamp(endTime, tz='Etc/GMT'), inplace=True)
        except:
            pass
        
        li.append(ts)
    
    if len(li)==0:
        
        print('no data found for all locations')
        dfTot = None
        return dfTot
    
    # concatenate dataframes of instruments
    dfTot = pd.concat(li, axis=0, ignore_index=False)
    
    # call empty column "COUNT"
    dfTot['qualifierId'].replace(r'^\s*$','COUNT',regex=True, inplace=True)
    dfTot['qualifierId'].replace(np.nan,'COUNT',inplace=True)
    dfTot['qualifierId'].replace("HN",'MIN',inplace=True)
    dfTot['qualifierId'].replace("HX",'MAX',inplace=True)
    
    # add columns with instrument name and short day string
    dfTot['instrument'] = instrument
    dfTot['day'] = daystr
    
    # add number of values per day
    dfTot['expected'] = N
    
    # add difference between values present and expected number (absolute and relative in %)
    dfTot.loc[(dfTot['qualifierId'] == 'COUNT'),'difference'] = dfTot.loc[(dfTot['qualifierId'] == 'COUNT'),'value']-dfTot.loc[(dfTot['qualifierId'] == 'COUNT'),'expected']
    dfTot.loc[(dfTot['qualifierId'] != 'COUNT'),'difference'] = dfTot.loc[(dfTot['qualifierId'] != 'COUNT'),'value']-0
    dfTot['diffprocent'] = np.round(np.abs(dfTot['difference']/dfTot['expected']*100))
    
    # flag differences larger than 5%
    dfTot['flag'] = 0
    dfTot.loc[(dfTot['qualifierId'] == 'COUNT') & (dfTot['diffprocent'] >= 5),'flag'] = 1
    dfTot.loc[(dfTot['qualifierId'] == 'MISSING') & (dfTot['diffprocent'] >= 1),'flag'] = 1
    dfTot.loc[(dfTot['qualifierId'] == 'MIN') & (dfTot['value'] != 0),'flag'] = 1
    dfTot.loc[(dfTot['qualifierId'] == 'MAX') & (dfTot['value'] != 0),'flag'] = 1
    
    # drop columns
    dfTot.drop(columns=['moduleInstanceId','units'], inplace=True)
    
    # remove duplicate counts
    dfTot = dfTot[(dfTot['qualifierId'] != 'COUNT') | (dfTot['parameterId'].values != 'N.S.0.east')]
    dfTot = dfTot[(dfTot['qualifierId'] != 'COUNT') | (dfTot['parameterId'].values != 'N.S.0.east.n02')]
    dfTot = dfTot[(dfTot['qualifierId'] != 'COUNT') | (dfTot['parameterId'].values != 'N.T.0')]
    
    # change order of columns
    cols = ['day','instrument','locationId','stationName','parameterId','qualifierId','value','expected','difference','diffprocent','flag']
    dfTot = dfTot[cols]
    
    return dfTot
